"use strict";
class DaptivStatsLogger {
    constructor(options) {
        this.prefix = this.sanitizeStatKey(options.prefix);
        if (!options.client) {
            throw new Error('Metrics will not be logged. No metrics client was provided to the DaptivStatsLogger');
        }
        this.client = options.client;
        this.globalTags = (options.globalTags || []).map(this.sanitizeStatKey);
    }
    timing(statKey, milliseconds, sampleRate, tags) {
        this.client.timing(this.decorateStatName(statKey), milliseconds, sampleRate || 1, this.mergeWithGlobalTags(tags));
    }
    increment(statKey, sampleRate, tags) {
        this.client.increment(this.decorateStatName(statKey), sampleRate || 1, this.mergeWithGlobalTags(tags));
    }
    incrementBy(statKey, value, tags) {
        this.client.incrementBy(this.decorateStatName(statKey), value, this.mergeWithGlobalTags(tags));
    }
    decrement(statKey, sampleRate, tags) {
        this.client.decrement(this.decorateStatName(statKey), sampleRate || 1, this.mergeWithGlobalTags(tags));
    }
    decrementBy(statKey, value, tags) {
        this.client.decrementBy(this.decorateStatName(statKey), value, this.mergeWithGlobalTags(tags));
    }
    gauge(statKey, value, sampleRate, tags) {
        this.client.gauge(this.decorateStatName(statKey), value, sampleRate || 1, this.mergeWithGlobalTags(tags));
    }
    histogram(statKey, value, sampleRate, tags) {
        this.client.histogram(this.decorateStatName(statKey), value, sampleRate || 1, this.mergeWithGlobalTags(tags));
    }
    close() {
        this.client.close();
    }
    mergeWithGlobalTags(tags) {
        return tags
            ? this.globalTags.concat(tags.map(this.sanitizeStatKey))
            : this.globalTags;
    }
    decorateStatName(statKey) {
        let sanitizedStatKey = this.sanitizeStatKey(statKey);
        return !this.prefix
            ? sanitizedStatKey
            : `${this.prefix}.${sanitizedStatKey}`;
    }
    sanitizeStatKey(statKey) {
        if (!statKey) {
            return statKey;
        }
        return statKey.toLowerCase().replace(/[^a-z0-9\.]/g, '_');
    }
}
exports.DaptivStatsLogger = DaptivStatsLogger;
